@@ -1,19 +1,20 @@
 import datetime
-import time
-from twisted.trial import unittest
-from twisted.internet import defer, task
-from twisted.python.failure import Failure
-from zope.interface import implements
+import ipaddress
 from mock import patch
 
-from txtorcon import Circuit
-from txtorcon import build_timeout_circuit
+from twisted.trial import unittest
+from twisted.internet import defer
+from twisted.python.failure import Failure
 
+from zope.interface import implementer
+
+from txtorcon import Circuit
 from txtorcon import Stream
 from txtorcon import TorControlProtocol
 from txtorcon import TorState
 from txtorcon import Router
 from txtorcon.router import hexIdFromHash
+from txtorcon.circuit import TorCircuitEndpoint, _get_circuit_attacher
 from txtorcon.interface import IRouterContainer
 from txtorcon.interface import ICircuitListener
 from txtorcon.interface import ICircuitContainer
@@ -23,8 +24,11 @@ from txtorcon.interface import ITorControlProtocol
 from mock import Mock
 
 
+@implementer(IRouterContainer)
+@implementer(ICircuitListener)
+@implementer(ICircuitContainer)
+@implementer(ITorControlProtocol)
 class FakeTorController(object):
-    implements(IRouterContainer, ICircuitListener, ICircuitContainer, ITorControlProtocol)
 
     post_bootstrap = defer.Deferred()
     queue_command = Mock()
@@ -81,6 +85,7 @@ class FakeRouter:
         self.id_hex = hexIdFromHash(self.id_hash)
         self.location = FakeLocation()
 
+
 examples = ['CIRC 365 LAUNCHED PURPOSE=GENERAL',
             'CIRC 365 EXTENDED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris PURPOSE=GENERAL',
             'CIRC 365 EXTENDED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus PURPOSE=GENERAL',
@@ -90,6 +95,82 @@ examples = ['CIRC 365 LAUNCHED PURPOSE=GENERAL',
             'CIRC 365 FAILED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL REASON=TIMEOUT']
 
 
+class TestCircuitEndpoint(unittest.TestCase):
+
+    @defer.inlineCallbacks
+    def test_attach(self):
+
+        @implementer(ICircuitContainer)
+        class FakeContainer(object):
+            pass
+
+        container = FakeContainer()
+        stream = Stream(container)
+        circuit = Mock()
+        target_endpoint = Mock()
+        reactor = Mock()
+        state = Mock()
+
+        TorCircuitEndpoint(
+            reactor, state, circuit, target_endpoint,
+        )
+
+        attacher = yield _get_circuit_attacher(reactor, state)
+        attacher.add_endpoint(target_endpoint, circuit)
+        yield attacher.attach_stream(stream, [])
+        # hmmm, no assert??
+
+    @defer.inlineCallbacks
+    def test_attach_stream_failure(self):
+
+        @implementer(ICircuitContainer)
+        class FakeContainer(object):
+            pass
+
+        container = FakeContainer()
+        stream = Stream(container)
+        stream.source_addr = ipaddress.IPv4Address(u'0.0.0.0')
+        stream.source_port = 12345
+        circuit = Mock()
+        circuit.when_built = Mock(return_value=Failure(Exception('testing1234')))
+        target_endpoint = Mock()
+        src_addr = Mock()
+        src_addr.host = u'0.0.0.0'
+        src_addr.port = 12345
+        target_endpoint._get_address = Mock(return_value=defer.succeed(src_addr))
+        reactor = Mock()
+        state = Mock()
+
+        TorCircuitEndpoint(
+            reactor, state, circuit, target_endpoint,
+        )
+
+        attacher = yield _get_circuit_attacher(reactor, state)
+        d = attacher.add_endpoint(target_endpoint, circuit)
+        self.assertEqual(len(attacher._circuit_targets), 1)
+        # this will fail, but should be ignored
+        yield attacher.attach_stream(stream, [])
+        with self.assertRaises(Exception) as ctx:
+            yield d
+        self.assertTrue("testing1234" in str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_attach_failure_unfound(self):
+
+        @implementer(ICircuitContainer)
+        class FakeContainer(object):
+            pass
+
+        reactor = Mock()
+        container = FakeContainer()
+        stream = Stream(container)
+        state = Mock()
+
+        attacher = yield _get_circuit_attacher(reactor, state)
+        attacher.attach_stream_failure(stream, None)
+        # no assert; just making sure this doesn't explode
+
+
 class CircuitTests(unittest.TestCase):
 
     def test_age(self):
@@ -103,7 +184,7 @@ class CircuitTests(unittest.TestCase):
         update = '1 LAUNCHED PURPOSE=GENERAL TIME_CREATED=%s' % now.strftime('%Y-%m-%dT%H:%M:%S')
         circuit.update(update.split())
         diff = circuit.age(now=now)
-        self.assertEquals(diff, 0)
+        self.assertEqual(diff, 0)
         self.assertTrue(circuit.time_created is not None)
 
     @patch('txtorcon.circuit.datetime')
@@ -119,7 +200,7 @@ class CircuitTests(unittest.TestCase):
 
         circuit = Circuit(tor)
         circuit._time_created = datetime.fromtimestamp(0.0)
-        self.assertEquals(circuit.age(), 60)
+        self.assertEqual(circuit.age(), 60)
         self.assertTrue(circuit.time_created is not None)
 
     def test_no_age_yet(self):
@@ -133,7 +214,7 @@ class CircuitTests(unittest.TestCase):
         circuit.update('1 LAUNCHED PURPOSE=GENERAL'.split())
         self.assertTrue(circuit.time_created is None)
         diff = circuit.age(now=now)
-        self.assertEquals(diff, None)
+        self.assertEqual(diff, None)
 
     def test_listener_mixin(self):
         listener = CircuitListenerMixin()
@@ -202,7 +283,8 @@ class CircuitTests(unittest.TestCase):
         circuit.update('1 CLOSED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL REASON=FINISHED'.split())
         circuit.update('1 FAILED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL REASON=TIMEOUT'.split())
         errs = self.flushLoggedErrors()
-        self.assertEqual(len(errs), 2)
+        self.assertEqual(len(errs), 1)
+        self.assertTrue('Circuit is FAILED but still has 1 streams' in str(errs[0]))
 
     def test_updates(self):
         tor = FakeTorController()
@@ -313,26 +395,38 @@ class CircuitTests(unittest.TestCase):
         circuit.update('123 EXTENDED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL'.split())
 
         self.assertEqual(3, len(circuit.path))
-        d = circuit.close()
+        d0 = circuit.close()
         # we already pretended that Tor answered "OK" to the
         # CLOSECIRCUIT call (see close_circuit() in FakeTorController
         # above) however the circuit isn't "really" closed yet...
-        self.assertTrue(not d.called)
+        self.assertTrue(not d0.result.called)
         # not unit-test-y? shouldn't probably delve into internals I
         # suppose...
         self.assertTrue(circuit._closing_deferred is not None)
 
+        # if we try to close it again (*before* the actual close has
+        # succeeded!) we should also still be waiting.
+        d1 = circuit.close()
+        self.assertTrue(not d1.called)
+        # ...and this Deferred should *not* be the same as the first
+        self.assertTrue(d0 is not d1)
+
         # simulate that Tor has really closed the circuit for us
         # this should cause our Deferred to callback
         circuit.update('123 CLOSED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL REASON=FINISHED'.split())
 
+        # if we close *after* the close has succeeded, then we should
+        # immediately "succeed"
+        d2 = circuit.close()
+        self.assertTrue(d1.called)
+
         # confirm that our circuit callback has been triggered already
         self.assertRaises(
             defer.AlreadyCalledError,
-            d.callback,
+            d0.callback,
             "should have been called already"
         )
-        return d
+        return defer.DeferredList([d0, d1, d2])
 
     def test_is_built(self):
         tor = FakeTorController()
@@ -369,6 +463,32 @@ class CircuitTests(unittest.TestCase):
         self.assertTrue(built1.result == circuit)
         self.assertTrue(built2.result == circuit)
 
+    def test_when_closed(self):
+        tor = FakeTorController()
+        a = FakeRouter('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a')
+        b = FakeRouter('$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5', 'b')
+        c = FakeRouter('$253DFF1838A2B7782BE7735F74E50090D46CA1BC', 'c')
+        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = a
+        tor.routers['$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5'] = b
+        tor.routers['$253DFF1838A2B7782BE7735F74E50090D46CA1BC'] = c
+
+        circuit = Circuit(tor)
+        circuit.listen(tor)
+
+        circuit.update('123 EXTENDED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL'.split())
+        d0 = circuit.when_closed()
+
+        self.assertFalse(d0.called)
+
+        circuit.update('123 BUILT $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL'.split())
+        circuit.update('123 CLOSED'.split())
+
+        d1 = circuit.when_closed()
+
+        self.assertTrue(d0 is not d1)
+        self.assertTrue(d0.called)
+        self.assertTrue(d1.called)
+
     def test_is_built_errback(self):
         tor = FakeTorController()
         a = FakeRouter('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a')
@@ -382,7 +502,41 @@ class CircuitTests(unittest.TestCase):
         state.circuit_new(circuit)
         d = circuit.when_built()
 
-        state.circuit_closed(circuit)
+        called = []
 
-        self.assertTrue(d.called)
-        self.assertTrue(isinstance(d.result, Failure))
+        def err(f):
+            called.append(f)
+            return None
+        d.addErrback(err)
+
+        state.circuit_closed(circuit, REASON='testing')
+
+        self.assertEqual(1, len(called))
+        self.assertTrue(isinstance(called[0], Failure))
+        self.assertTrue('testing' in str(called[0].value))
+        return d
+
+    def test_stream_success(self):
+        tor = FakeTorController()
+        a = FakeRouter('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a')
+        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = a
+
+        circuit = Circuit(tor)
+        reactor = Mock()
+
+        circuit.stream_via(
+            reactor, 'torproject.org', 443,
+            Mock(),
+            use_tls=True,
+        )
+
+    def test_circuit_web_agent(self):
+        tor = FakeTorController()
+        a = FakeRouter('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a')
+        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = a
+
+        circuit = Circuit(tor)
+        reactor = Mock()
+
+        # just testing this doesn't cause an exception
+        circuit.web_agent(reactor, Mock())
