@@ -64,6 +64,37 @@ class MockReactor(Mock):
     pass
 
 
+def fake_tor_launcher(config):
+    # exactly match the signature of txtorcon.controller.launch
+    def launch(reactor,
+               progress_updates=None,
+               control_port=None,
+               data_directory=None,
+               socks_port=None,
+               non_anonymous_mode=None,
+               stdout=None,
+               stderr=None,
+               timeout=None,
+               tor_binary=None,
+               user=None,
+               connection_creator=None,
+               kill_on_stderr=True,
+               _tor_config=None,
+               ):
+        return defer.succeed(Tor(reactor, None, config, None, non_anonymous_mode))
+
+    return launch
+
+
+def valid_enough_torconfig():
+    config = TorConfig()
+    # The implementation will probably ask the configuration about the control
+    # port to make sure it agrees with the control port of the global Tor
+    # instance it believes will be used.  Give it an answer.
+    config.ControlPort = 12345
+    return config
+
+
 @patch('txtorcon.controller.find_tor_binary', return_value='/bin/echo')
 class EndpointTests(unittest.TestCase):
 
@@ -72,6 +103,7 @@ class EndpointTests(unittest.TestCase):
         endpoints._global_tor_config = None
         del endpoints._global_tor_lock
         endpoints._global_tor_lock = defer.DeferredLock()
+        endpoints._global_tor = None
         self.reactor = FakeReactorTcp(self)
         self.protocol = FakeControlProtocol([])
         self.protocol.event_happened('INFO', 'something craaaaaaazy')
@@ -103,17 +135,18 @@ class EndpointTests(unittest.TestCase):
     def tearDown(self):
         from txtorcon import endpoints
         endpoints._global_tor_config = None
+        endpoints._global_tor = None
+        assert not endpoints._global_tor_lock.locked, "Should not be locked a teardown"
         del endpoints._global_tor_lock
         endpoints._global_tor_lock = defer.DeferredLock()
+        endpoints._global_tor = None
         self.patcher.stop()
 
     @defer.inlineCallbacks
     def test_global_tor(self, ftb):
-        fake_tor = Mock()
-        fake_tor.get_config = Mock(return_value=self.config)
         config = yield get_global_tor(
             Mock(),
-            _tor_launcher=lambda x, progress_updates=None: fake_tor
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         # XXX this was asserting SOCKSPort == 0 before; why?
         self.assertEqual(['9050'], config.SOCKSPort)
@@ -122,7 +155,7 @@ class EndpointTests(unittest.TestCase):
     def test_global_tor_error(self, ftb):
         yield get_global_tor(
             reactor=FakeReactorTcp(self),
-            _tor_launcher=lambda x, y, progress_updates=None: True
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         # now if we specify a control_port it should be an error since
         # the above should have launched one.
@@ -130,7 +163,7 @@ class EndpointTests(unittest.TestCase):
             yield get_global_tor(
                 reactor=FakeReactorTcp(self),
                 control_port=111,
-                _tor_launcher=lambda x, y, progress_updates=None: True
+                _tor_launcher=fake_tor_launcher(self.config),
             )
             self.fail()
         except RuntimeError:
@@ -545,7 +578,7 @@ class EndpointTests(unittest.TestCase):
         control_ep.connect = Mock(return_value=defer.succeed(None))
         directlyProvides(control_ep, IStreamClientEndpoint)
         ep = TCPHiddenServiceEndpoint.system_tor(self.reactor, control_ep, 1234)
-        ep._tor_progress_update(40, "FOO", "foo to bar")
+        ep._tor_progress_update(40, "FOO", "foo to the bar")
         return ep
 
     def test_single_hop_non_ephemeral(self, ftb):
@@ -701,13 +734,9 @@ class EndpointTests(unittest.TestCase):
     def test_parse_via_plugin(self, ftb):
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         ep = serverFromString(
             self.reactor,
@@ -725,13 +754,9 @@ class EndpointTests(unittest.TestCase):
 
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         ep = serverFromString(
             self.reactor,
@@ -749,13 +774,9 @@ class EndpointTests(unittest.TestCase):
 
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         ep = serverFromString(
             self.reactor,
@@ -777,13 +798,9 @@ class EndpointTests(unittest.TestCase):
 
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         ep = serverFromString(
             self.reactor,
@@ -804,13 +821,9 @@ class EndpointTests(unittest.TestCase):
 
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
 
         with self.assertRaises(ValueError):
@@ -827,13 +840,9 @@ class EndpointTests(unittest.TestCase):
 
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         ep = serverFromString(
             self.reactor,
@@ -852,13 +861,9 @@ class EndpointTests(unittest.TestCase):
 
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         ep = serverFromString(
             self.reactor,
@@ -913,11 +918,12 @@ class EndpointTests(unittest.TestCase):
         )
 
     def test_parse_version_3(self, ftb):
-        ep = serverFromString(
-            self.reactor,
-            'onion:88:version=3:localPort=1234:hiddenServiceDir=~/blam/blarg'
-        )
-        self.assertFalse(ep.ephemeral)
+        with patch('txtorcon.endpoints.get_global_tor_instance'):
+            ep = serverFromString(
+                self.reactor,
+                'onion:88:version=3:localPort=1234:hiddenServiceDir=~/blam/blarg'
+            )
+            self.assertFalse(ep.ephemeral)
 
     def test_parse_user_path(self, ftb):
         # this makes sure we expand users and symlinks in
@@ -925,13 +931,9 @@ class EndpointTests(unittest.TestCase):
 
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
         ep = serverFromString(
             self.reactor,
@@ -951,13 +953,9 @@ class EndpointTests(unittest.TestCase):
 
         # make sure we have a valid thing from get_global_tor without
         # actually launching tor
-        config = TorConfig()
-        config.post_bootstrap = defer.succeed(config)
-        from txtorcon import torconfig
-        torconfig._global_tor_config = None
         get_global_tor(
             self.reactor,
-            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+            _tor_launcher=fake_tor_launcher(self.config),
         )
 
         orig = os.path.realpath('.')
@@ -988,7 +986,7 @@ class EndpointTests(unittest.TestCase):
         with self.assertRaises(ValueError) as ctx:
             TCPHiddenServiceEndpoint(
                 reactor, config, 80,
-                stealth_auth=['alice', 'bob'],
+                auth=AuthStealth(['alice', 'bob']),
                 ephemeral=True,
             )
         self.assertIn(
